@@ -54,6 +54,9 @@ typedef struct _sPreviousValues1stOrder {
     Float32	yr_1;
 } PreviousValues1stOrder;
 
+
+float gOldSample;	// [3134221] aml
+
 // aml 4.30.02
 // -24 dB to +24 dB in 1 dB steps
 const UInt16 kZeroGaindBConvTableOffset = 24;
@@ -281,28 +284,51 @@ IOReturn clipAppleDBDMAToOutputStream(const void *mixBuf, void *sampleBuf, UInt3
 		}
 	}        
 
+    return kIOReturnSuccess;
+}
+
+// [3134221] aml
+IOReturn clipAppleDBDMAToOutputStreamDelayRight(const void *mixBuf, void *sampleBuf, UInt32 firstSampleFrame, UInt32 numSampleFrames, const IOAudioStreamFormat *streamFormat)
+{
+
+    UInt32 		sampleIndex, maxSampleIndex;
+    float *		floatMixBuf;
+    SInt16 *	outputBuf;
     
-//    floatMixBuf = (float *)mixBuf;
-//    outputBuf = (SInt16 *)sampleBuf; 
-
-/*    maxSampleIndex = (firstSampleFrame + numSampleFrames) * streamFormat->fNumChannels;
-
-    for (sampleIndex = (firstSampleFrame * streamFormat->fNumChannels); sampleIndex < maxSampleIndex; sampleIndex++) {
-        float inSample;
-
-        inSample = floatMixBuf[sampleIndex];
-        if (inSample > 1.0) {
-                inSample = 1.0;
-        } else if (inSample < -1.0) {
-                inSample = -1.0;
+    floatMixBuf = (float *)mixBuf;
+    outputBuf = (SInt16 *)sampleBuf;
+  
+    maxSampleIndex = (firstSampleFrame + numSampleFrames) * streamFormat->fNumChannels;
+    
+    for (sampleIndex = (firstSampleFrame * streamFormat->fNumChannels); sampleIndex < maxSampleIndex; sampleIndex += 2) 
+    {
+        float inSampleL, inSampleR;
+        
+        inSampleL = floatMixBuf[sampleIndex];
+		inSampleR = gOldSample;
+		gOldSample = floatMixBuf[sampleIndex + 1];
+		
+        if (inSampleL > 1.0) 
+        {
+            inSampleL = 1.0;
+        }
+        else if (inSampleL < -1.0) 
+        {
+            inSampleL = -1.0;
         }
 
-        if (inSample >= 0) {
-                outputBuf[sampleIndex] = (SInt16) (inSample * 32767.0);
-        } else {
-                outputBuf[sampleIndex] = (SInt16) (inSample * 32767.0);
+        if (inSampleR > 1.0) 
+        {
+            inSampleR = 1.0;
         }
-    }*/
+        else if (inSampleR < -1.0) 
+        {
+            inSampleR = -1.0;
+        }
+        
+		outputBuf[sampleIndex] = (SInt16) (inSampleL * 32767.0);
+		outputBuf[sampleIndex + 1] = (SInt16) (inSampleR * 32767.0);
+    }
 
     return kIOReturnSuccess;
 }
@@ -525,6 +551,162 @@ IOReturn clipAppleDBDMAToOutputStreamiSub(const void *mixBuf, void *sampleBuf, P
     }
     return kIOReturnSuccess;
 }
+
+// [3134221] aml
+IOReturn clipAppleDBDMAToOutputStreamiSubDelayRight(const void *mixBuf, void *sampleBuf, PreviousValues * filterState, PreviousValues * filterState2, PreviousValues * phaseCompState, Float32 *low, Float32 *high, UInt32 firstSampleFrame, UInt32 numSampleFrames, UInt32 sampleRate, const IOAudioStreamFormat *streamFormat, SInt16 *iSubBufferMemory, UInt32 *loopCount, SInt32 *iSubBufferOffset, UInt32 iSubBufferLen, iSubAudioFormatType* iSubFormat, float* srcPhase, float* srcState, UInt32 adaptiveSampleRate)
+{
+    UInt32 sampleIndex, maxSampleIndex;
+    float *	floatMixBuf;
+    SInt16 *	outputBuf;
+    Float32	iSubSampleFloat;
+    SInt16	iSubSampleInt;
+    // aml 3.6.02 storage for src
+    float 	x0, x1, temp;
+
+    // aml 3.6.02 src variables - should calculate phaseInc somewhere else, change only if the SR changes
+    float 	phaseInc = ((float)adaptiveSampleRate)/((float)(iSubFormat->outputSampleRate));		// phase increment = Fs_in/Fs_out
+    float 	phase = *srcPhase;								// current phase location
+
+    floatMixBuf = (float *)mixBuf;
+    outputBuf = (SInt16 *)sampleBuf;
+ 
+    maxSampleIndex = (firstSampleFrame + numSampleFrames) * streamFormat->fNumChannels;
+
+    // Filter out the highs and lows for use with the iSub
+    if (1 == streamFormat->fNumChannels) {
+		MonoFilter (&floatMixBuf[firstSampleFrame * streamFormat->fNumChannels], &low[firstSampleFrame * streamFormat->fNumChannels], &high[firstSampleFrame * streamFormat->fNumChannels], numSampleFrames, sampleRate);
+    } else if (2 == streamFormat->fNumChannels) {
+		// aml 2.15.02 changed to 4th order version
+		// aml 2.18.02 changed to 4th order version with phase compensation
+		StereoFilter4thOrderPhaseComp (&floatMixBuf[firstSampleFrame * streamFormat->fNumChannels], &low[firstSampleFrame * streamFormat->fNumChannels], &high[firstSampleFrame * streamFormat->fNumChannels], numSampleFrames, sampleRate, filterState, filterState2, phaseCompState);
+    }
+    //
+    // high side loop 
+    //
+    for (sampleIndex = (firstSampleFrame * streamFormat->fNumChannels); sampleIndex < maxSampleIndex; sampleIndex += 2) {
+
+
+        float inSampleL, inSampleR;
+        
+        inSampleL = high[sampleIndex];
+		inSampleR = gOldSample;
+		gOldSample = high[sampleIndex + 1];
+		
+        if (inSampleL > 1.0) 
+        {
+            inSampleL = 1.0;
+        }
+        else if (inSampleL < -1.0) 
+        {
+            inSampleL = -1.0;
+        }
+
+        if (inSampleR > 1.0) 
+        {
+            inSampleR = 1.0;
+        }
+        else if (inSampleR < -1.0) 
+        {
+            inSampleR = -1.0;
+        }
+        
+		outputBuf[sampleIndex] = (SInt16) (inSampleL * 32767.0);
+		outputBuf[sampleIndex + 1] = (SInt16) (inSampleR * 32767.0);
+    }
+    
+    // aml 2.28.02 added new loop for mixing iSub data to mono
+    // increment by two, mix each pair of samples
+    // aml 3.1.02 added iSub format check
+    //
+    // low side loops - currently only handles stereo->stereo, mono->mono, and stereo->mono
+    //
+    // aml XXX: should move this into an inline function - there's too much code copying!
+    if ((iSubFormat->numChannels == 1) && (streamFormat->fNumChannels == 2)) {
+        // aml 3.6.02 linear interpolation src (takes the edge of the zoh version, without wasting too many 
+        // cycles since we have a 4th order lp in front of us, down -90 dB at Nyquist for 6kHz sample rate)
+        // STEREO->MONO
+        sampleIndex = (firstSampleFrame * streamFormat->fNumChannels);
+        while (sampleIndex < maxSampleIndex) {
+                    
+            if (phase >= 1.0) {	
+                phase -= 1.0;
+                sampleIndex+=2;
+            } else {   
+                // check for beginning of frame case, use saved last sample if needed
+                if (sampleIndex == (firstSampleFrame * streamFormat->fNumChannels)) { 
+                    x0 = *srcState;
+                } else {
+                    // mix x[n-1] to mono
+                    x0 = low[sampleIndex-2];
+                    temp = low[sampleIndex-1];
+                    x0 = 0.5*(x0 + temp);
+                }
+                
+                // mix x[n] to mono
+                x1 = low[sampleIndex];
+                temp = low[sampleIndex+1];
+                x1 = 0.5*(x1 + temp);
+                                
+                // linearly interpolate between x0 and x1
+                iSubSampleFloat = x0 + phase*(x1 - x0);
+                
+                // clip
+                if (iSubSampleFloat > 1.0) {
+                    iSubSampleFloat = 1.0;
+                } else if (iSubSampleFloat < -1.0) {
+                    iSubSampleFloat = -1.0;
+                }
+                
+                // convert to fixed
+                iSubSampleInt = (SInt16) (iSubSampleFloat * 32767.0);
+                
+                // check for end of buffer condition
+                if (*iSubBufferOffset >= (SInt32)iSubBufferLen) {
+                    *iSubBufferOffset = 0;
+                    (*loopCount)++;
+                }
+                
+                // byteswap to USB format and copy to iSub buffer
+                iSubBufferMemory[(*iSubBufferOffset)++] = ((((UInt16)iSubSampleInt) << 8) & 0xFF00) | ((((UInt16)iSubSampleInt) >> 8) & 0x00FF);
+
+                // increment phase and update input buffer pointer
+                phase += phaseInc;		
+            }
+		}
+        if (phase < 1) {
+            // mix and save last sample in buffer to mono if it will be needed for the next loop
+            x1 = low[maxSampleIndex-2];
+            temp = low[maxSampleIndex-1];
+            *srcState = 0.5*(x1 + temp);
+        } else {
+            *srcState = 0;
+        }
+        // cache current phase for use next time we enter the clip loop
+       *srcPhase = phase;
+    } else {
+        // STEREO->STEREO, MONO->MONO
+        for (sampleIndex = (firstSampleFrame * streamFormat->fNumChannels); sampleIndex < maxSampleIndex; sampleIndex++) {
+                    
+            iSubSampleFloat = low[sampleIndex];
+            if (iSubSampleFloat > 1.0) {
+				iSubSampleFloat = 1.0;
+            } else if (iSubSampleFloat < -1.0) {
+				iSubSampleFloat = -1.0;
+            }
+
+            iSubSampleInt = (SInt16) (iSubSampleFloat * 32767.0);
+
+            if (*iSubBufferOffset >= iSubBufferLen) {
+				*iSubBufferOffset = 0;
+				(*loopCount)++;
+            }
+
+            iSubBufferMemory[(*iSubBufferOffset)++] = ((((UInt16)iSubSampleInt) << 8) & 0xFF00) | ((((UInt16)iSubSampleInt) >> 8) & 0x00FF);
+        }
+    }
+    return kIOReturnSuccess;
+}
+
 // aml 2.14.02 added second filter state for 4th order filter
 // aml 2.18.02 changed to 4th order version with phase compensation
 // aml 3.5.02 added src phase parameter
