@@ -16,6 +16,9 @@
 
 #define super OSObject
 
+// aml 5.27.02
+#define OVERRIDE_REGISTRY_ZERO_GAIN	1
+
 OSDefineMetaClassAndStructors(AudioDeviceTreeParser, OSObject)
 
 
@@ -43,7 +46,7 @@ bool AudioDeviceTreeParser::init(IOService *provider){
     if(!super::init())
         return(false);
     
-    soundEntry =  provider->childFromPath(kSoundEntryName, gIODTPlane);               
+    soundEntry = provider->childFromPath(kSoundEntryName, gIODTPlane);               
     if(!soundEntry) return false;
     
     soundEntry->retain();
@@ -94,10 +97,10 @@ OSArray *AudioDeviceTreeParser::createOutputsArray(){
     FAIL_IF(!tempData, BAIL);
 
     theSoundObjects = (char *) tempData->getBytesNoCopy();
-    size = (int) tempData->getCapacity();
+    size = (int) tempData->getLength();
     parser = -1;
     
-    while(parser < size) {
+    while(size && parser < size) {
 
         // find the end of string
         ASSIGNSTARTSTRING(startidx, parser);
@@ -107,7 +110,7 @@ OSArray *AudioDeviceTreeParser::createOutputsArray(){
                         
 		// do the detect parsing if it is one
         stopwordidx = 0;
-        NEXTENDOFWORD(thetempObject, stopwordidx);
+        NEXTENDOFWORD(thetempObject, stopwordidx, size);
                         
         if(!bcmp(thetempObject, kOutputObjEntryName, stopwordidx-1)) {
             length = stopidx-startidx;
@@ -115,39 +118,39 @@ OSArray *AudioDeviceTreeParser::createOutputsArray(){
             
             while(stopwordidx < length) {
 
-                NEXTENDOFWORD(thetempObject, stopwordidx);
+                NEXTENDOFWORD(thetempObject, stopwordidx, size);
                 
                 if(!bcmp(thetempObject+startwordidx, kDeviceMaskPropName, stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);
                     READWORDASNUMBER(myInfo.deviceMask, thetempObject, startwordidx, stopwordidx);
                 } else if(!bcmp(thetempObject+startwordidx, kDeviceMatchPropName, stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);
                     READWORDASNUMBER(myInfo.deviceMatch, thetempObject, startwordidx, stopwordidx);
                 } else if(!bcmp(thetempObject+startwordidx, kIconIDPropName, stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);    
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);    
                     READWORDASNUMBER(myInfo.iconID, thetempObject, startwordidx, stopwordidx);
                 } else if(!bcmp(thetempObject+startwordidx, kPortConnectionPropName, stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);    
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);    
                     READWORDASNUMBER(myInfo.portConnection, thetempObject, startwordidx, stopwordidx);
                 } else if(!bcmp(thetempObject+startwordidx, kPortTypePropName, stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx); 
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size); 
                     READWORDASNUMBER(myInfo.portType, thetempObject, startwordidx, stopwordidx);
                 } else if(!bcmp(thetempObject+startwordidx, kNameIDPropName, stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);    
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);    
                     READWORDASNUMBER(myInfo.nameID, thetempObject, startwordidx, stopwordidx);
                 } else if(!bcmp(thetempObject+startwordidx, "param", stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);    
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);    
                     READWORDASNUMBER(myInfo.param, thetempObject, startwordidx, stopwordidx);
                 } else if(!bcmp(thetempObject+startwordidx, kModelPropName, stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx); 
-                    NEXTENDOFWORD(thetempObject, stopwordidx);
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);
                     
                     if(!bcmp(thetempObject+startwordidx, kOutputPortObjName, 8))
                         myInfo.outputKind = kOutputPortTypeClassic;
@@ -179,8 +182,6 @@ BAIL:
     goto EXIT;
 }
 
-
-
 UInt32 AudioDeviceTreeParser::getNumberOfInputs(){
     UInt32 result = 0;
     OSData *tempData;
@@ -207,10 +208,10 @@ UInt32 AudioDeviceTreeParser::getNumberofInputsWithMuxes(){
     FAIL_IF(!tempData, BAIL);
 
     theSoundObjects = (char *) tempData->getBytesNoCopy();
-    size = (int) tempData->getCapacity();
+    size = (int) tempData->getLength();
     parser = -1;
 
-    while(parser < size) {
+    while(size && parser < size) {
         ASSIGNSTARTSTRING(startidx, parser);
         NEXTENDOFSTRING(theSoundObjects, parser);
         ASSIGNSTOPSTRING(stopidx, parser);
@@ -218,7 +219,7 @@ UInt32 AudioDeviceTreeParser::getNumberofInputsWithMuxes(){
                         
 		// do the detect parsing if it is one
         stopwordidx = 0;
-        NEXTENDOFWORD(thetempObject, stopwordidx);
+        NEXTENDOFWORD(thetempObject, stopwordidx, size);
         if(!bcmp(thetempObject, kInputObjEntryName, stopwordidx-1)) 
             result++;
         else if( !bcmp(thetempObject, kMuxObjEntryName, stopwordidx-1))
@@ -232,7 +233,6 @@ BAIL:
     goto EXIT;
 }
 
-
 UInt32 AudioDeviceTreeParser::getNumberOfDetects(){
     UInt32 result = 0;
     OSData *tempData;
@@ -252,7 +252,6 @@ UInt32 AudioDeviceTreeParser::getNumberOfFeatures(){
     UInt32 result = 0;
  
     return result;
-
 }
 
 SInt16 AudioDeviceTreeParser::getInitOperationType(){
@@ -268,10 +267,11 @@ SInt16 AudioDeviceTreeParser::getInitOperationType(){
     FAIL_IF(!tempData, BAIL);
 
     theSoundObjects = (char *) tempData->getBytesNoCopy();
-    size = (int) tempData->getCapacity();
+	FailIf (NULL == theSoundObjects, BAIL);
+    size = (int) tempData->getLength();
     parser = -1;
     
-    while(parser < size) {
+    while(size && parser < size) {
 
 		// find the end of string
         ASSIGNSTARTSTRING(startidx, parser);
@@ -279,29 +279,29 @@ SInt16 AudioDeviceTreeParser::getInitOperationType(){
         ASSIGNSTOPSTRING(stopidx, parser);
         thetempObject = theSoundObjects+startidx;
                         
-		// do the detect parsing if it is one
+		// do the detect parsing if there is one
         stopwordidx = 0;
-        NEXTENDOFWORD(thetempObject, stopwordidx);
+        NEXTENDOFWORD(thetempObject, stopwordidx, size);
                         
         if(!bcmp(thetempObject, "init", stopwordidx-1)) {
             length = stopidx-startidx;
             startwordidx = ++stopwordidx;
             
             while(stopwordidx < length) {
-                NEXTENDOFWORD(thetempObject, stopwordidx);
+                NEXTENDOFWORD(thetempObject, stopwordidx, size);
                 
                 if(!bcmp(thetempObject+startwordidx, "operation", stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);
                     READWORDASNUMBER(opkind, thetempObject, startwordidx, stopwordidx);
                 } else if(!bcmp(thetempObject+startwordidx, "param", stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);
                     thetempObject[stopwordidx] = '\0';
                     thetempObject[stopwordidx] = ' ';
                 } else if(!bcmp(thetempObject+startwordidx, "param-size", stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);    
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);    
                     thetempObject[stopwordidx] = '\0';
                     thetempObject[stopwordidx] = ' ';
                 }                 
@@ -318,7 +318,6 @@ BAIL:
     goto EXIT;
 }
 
-
 // cheap and dirty way to get the phase inversion feature object.
 // Do it this way until we parse all the features directly
 bool AudioDeviceTreeParser::getPhaseInversion()
@@ -334,12 +333,11 @@ bool AudioDeviceTreeParser::getPhaseInversion()
     FAIL_IF(!tempData, BAIL);
 
     theSoundObjects = (char *) tempData->getBytesNoCopy();
-    size = (int) tempData->getCapacity();
+    size = (int) tempData->getLength();
     parser = -1;
     
-    while(parser < size) 
+    while(size && parser < size) 
     {
-
 		// find the end of string
         ASSIGNSTARTSTRING(startidx, parser);
         NEXTENDOFSTRING(theSoundObjects, parser);
@@ -348,7 +346,7 @@ bool AudioDeviceTreeParser::getPhaseInversion()
                         
 		// do the feature  parsing if it is one
         stopwordidx = 0;
-        NEXTENDOFWORD(thetempObject, stopwordidx);
+        NEXTENDOFWORD(thetempObject, stopwordidx, size);
                         
         if(!bcmp(thetempObject, "feature", stopwordidx-1)) 
         {
@@ -357,12 +355,12 @@ bool AudioDeviceTreeParser::getPhaseInversion()
             
             while(stopwordidx < length) 
             {
-                NEXTENDOFWORD(thetempObject, stopwordidx);
+                NEXTENDOFWORD(thetempObject, stopwordidx, size);
                 
                 if(!bcmp(thetempObject+startwordidx, "model", stopwordidx-startwordidx)) 
                 {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);
                     if(!bcmp(thetempObject+startwordidx, "PhaseInversion", 8)) 
                     {
                         result = true;
@@ -395,11 +393,10 @@ SInt16 AudioDeviceTreeParser::getPowerObjectType(){
     FAIL_IF(!tempData, BAIL);
 
     theSoundObjects = (char *) tempData->getBytesNoCopy();
-    size = (int) tempData->getCapacity();
+    size = (int) tempData->getLength();
     parser = -1;
     
-    while(parser < size) {
-
+    while(size && parser < size) {
 		// find the end of string
         ASSIGNSTARTSTRING(startidx, parser);
         NEXTENDOFSTRING(theSoundObjects, parser);
@@ -408,18 +405,18 @@ SInt16 AudioDeviceTreeParser::getPowerObjectType(){
                         
 		// do the feature  parsing if it is one
         stopwordidx = 0;
-        NEXTENDOFWORD(thetempObject, stopwordidx);
+        NEXTENDOFWORD(thetempObject, stopwordidx, size);
                         
         if(!bcmp(thetempObject, "feature", stopwordidx-1)) {
             length = stopidx-startidx;
             startwordidx = ++stopwordidx;
             
             while(stopwordidx < length) {
-                NEXTENDOFWORD(thetempObject, stopwordidx);
+                NEXTENDOFWORD(thetempObject, stopwordidx, size);
                 
                 if(!bcmp(thetempObject+startwordidx, "model", stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);
                     if(!bcmp(thetempObject+startwordidx, "Proj7PowerControl", 8)) 
                         result= kProj7PowerObject;
                     else if (!bcmp(thetempObject+startwordidx, "Proj10PowerControl", 8)) 
@@ -446,7 +443,6 @@ BAIL:
     result = 0;   
 }
 
-
 UInt32 AudioDeviceTreeParser::getLayoutID(){
     UInt32 result = 0;
     OSData *tempData;
@@ -462,18 +458,172 @@ BAIL:
     goto EXIT;
 }
 
-
 OSArray *AudioDeviceTreeParser::getDSPFeatures(){
  
  return(0);
 }
 
-
 OSArray *AudioDeviceTreeParser::createInputsArray(){
 	// this get the inputs, not used because handles everything.
 	return(0);
 }
 
+//
+// aml 4.29.02, returns true if we have HW input gain by checking the current HW model
+//
+bool AudioDeviceTreeParser::getHasHWInputGain(){
+    bool hasHWInputGain = true;
+    OSData *tempData;
+    
+    tempData = OSDynamicCast(OSData, soundEntry->getProperty(kModelPropName));
+    FAIL_IF(!tempData, EXIT);
+
+    if((!bcmp(tempData->getBytesNoCopy(), kDacaModelName, tempData->getLength())) || 
+        (!bcmp(tempData->getBytesNoCopy(), kTexasModelName, tempData->getLength())) || 
+        (!bcmp(tempData->getBytesNoCopy(), kTexas2ModelName, tempData->getLength()))) {
+		hasHWInputGain = false;
+    } else {
+		hasHWInputGain = true;
+    }
+
+// aml XXX testing
+#if 0    
+    if (!bcmp(tempData->getBytesNoCopy(), kDacaModelName, tempData->getLength())) {
+        IOLog("AudioDeviceTreeParser::getHasHWInputGain - kDacaModelName.\n");
+    } else if (!bcmp(tempData->getBytesNoCopy(), kTexasModelName, tempData->getLength())) {
+        IOLog("AudioDeviceTreeParser::getHasHWInputGain - kTexasModelName.\n");
+    } else if (!bcmp(tempData->getBytesNoCopy(), kTexas2ModelName, tempData->getLength())) {
+        IOLog("AudioDeviceTreeParser::getHasHWInputGain - kTexas2ModelName.\n");
+    } else if (!bcmp(tempData->getBytesNoCopy(), kAWACsModelName, tempData->getLength())) {
+        IOLog("AudioDeviceTreeParser::getHasHWInputGain - kAWACsModelName.\n");
+    } else if (!bcmp(tempData->getBytesNoCopy(), kScreamerModelName, tempData->getLength())) {
+        IOLog("AudioDeviceTreeParser::getHasHWInputGain - kScreamerModelName.\n");
+    } else if (!bcmp(tempData->getBytesNoCopy(), kBurgundyModelName, tempData->getLength())) {
+        IOLog("AudioDeviceTreeParser::getHasHWInputGain - kBurgundyModelName.\n");
+    } 
+    IOLog("AudioDeviceTreeParser::getHasHWInputGain - has HW gain = %d.\n", hasHWInputGain);
+#endif
+
+EXIT:
+    return hasHWInputGain;
+}
+
+//
+// aml 4.26.02, returns a dictionary of types (imic, line, etc.) and their zero dB gain offsets
+//
+UInt32 AudioDeviceTreeParser::getInternalMicGainOffset(){
+    UInt32 zeroGain;
+
+	zeroGain = 0;
+#if OVERRIDE_REGISTRY_ZERO_GAIN
+	// override the registry entries with these fixed values.
+	// values come from open firmware files and OS 9 source code.
+	// see AudioHardwareConstants.h for the full list of layouts.
+	switch (getLayoutID()) {
+		case layout101:
+			zeroGain = 0x00090000;	// 9.0 dB
+			break;
+		case layoutKihei:
+			zeroGain = 0x001E0000;	// 30 dB
+			break;
+		case layoutPismo:
+			zeroGain = 0x00138000;	// 19.5 dB 
+			break;
+		case layoutMercury:
+			zeroGain = 0x00090000;	// 9.0 dB 
+			break;
+		case layoutPerigee:
+			zeroGain = 0x001E0000;	// 30 dB
+			break;
+		case layoutWallStreet:
+			zeroGain = 0x00090000;	// 9.0 dB
+			break;
+		default:
+			zeroGain = 0x0;			// 0 dB
+			break;
+	}
+
+#else // OVERRIDE_REGISTRY_ZERO_GAIN
+
+	OSData *			tempData;
+	UInt32				portType;
+	char *				theSoundObjects;
+	char *				thetempObject;
+	int					parser;
+	int					startidx;
+	int					stopidx;
+	int					stopwordidx;
+	int					startwordidx;
+	int					size;
+	int					length;
+	UInt32				result;
+
+    result = 0;
+    tempData = OSDynamicCast(OSData, soundEntry->getProperty(kSoundObjectsPropName));
+    FAIL_IF(!tempData, BAIL);
+
+    theSoundObjects = (char *) tempData->getBytesNoCopy();
+    size = (int) tempData->getLength();
+    parser = -1;
+
+    while(size && parser < size) {
+        ASSIGNSTARTSTRING(startidx, parser);
+        NEXTENDOFSTRING(theSoundObjects, parser);
+        ASSIGNSTOPSTRING(stopidx, parser);
+        thetempObject = theSoundObjects+startidx;
+                                                
+        stopwordidx = 0;
+        NEXTENDOFWORD(thetempObject, stopwordidx);
+        //
+        // Parse for input or mux objects
+        //
+        if(!bcmp(thetempObject, kInputObjEntryName, stopwordidx-1) ||
+		   !bcmp(thetempObject, kMuxObjEntryName, stopwordidx-1)) {
+		   
+            length = stopidx-startidx;
+            startwordidx = ++stopwordidx;
+            
+            while(stopwordidx < length) {
+                //
+                // reset variables
+                //
+                portType = 0;
+
+                NEXTENDOFWORD(thetempObject, stopwordidx);
+                //
+                // Parse input objects for input port or mux type (internal mic, external mic, line, etc.)
+                //
+                if(!bcmp(thetempObject+startwordidx, kPortTypePropName, stopwordidx-startwordidx)) {
+                    ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
+                    NEXTENDOFWORD(thetempObject, stopwordidx); 
+                    READWORDASNUMBER(portType, thetempObject, startwordidx, stopwordidx);
+
+                    NEXTENDOFWORD(thetempObject, stopwordidx);
+                     
+                    if (portType == kIntMicSource) {
+                        //
+                        // next word is the zero gain value
+                        //
+                        ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
+                      	NEXTENDOFWORD(thetempObject, stopwordidx); 
+                        if(!bcmp(thetempObject+startwordidx, kZeroGainPropName, stopwordidx-startwordidx)) {
+                            ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
+                            NEXTENDOFWORD(thetempObject, stopwordidx); 
+                            READWORDASNUMBER(zeroGain, thetempObject, startwordidx, stopwordidx);
+						}
+                    }
+                    ASSIGNNEXTWORD(thetempObject, startwordidx, stopwordidx);                                                 
+                }                
+                ASSIGNNEXTWORD(thetempObject, startwordidx, stopwordidx);                                                 
+            }
+        } 
+    }
+
+#endif // OVERRIDE_REGISTRY_ZERO_GAIN
+
+    return zeroGain;
+}
+
 OSArray *AudioDeviceTreeParser::createInputsArrayWithMuxes(){
     // this get the inputs and the muxes there are on
     OSArray *AudioInputs = 0;
@@ -504,14 +654,13 @@ OSArray *AudioDeviceTreeParser::createInputsArrayWithMuxes(){
     FAIL_IF(!tempData, BAIL);
 
     theSoundObjects = (char *) tempData->getBytesNoCopy();
-    size = (int) tempData->getCapacity();
+    size = (int) tempData->getLength();
     parser = -1;
     
     myInfo.sndHWPort = 0; myInfo.inputPortType = 0;
     myInfo.channels = 0;   myInfo.isOnMuX = muxflag;
 
-    while(parser < size) {
-
+    while(size && parser < size) {
 		// find the end of string
         ASSIGNSTARTSTRING(startidx, parser);
         NEXTENDOFSTRING(theSoundObjects, parser);
@@ -520,7 +669,7 @@ OSArray *AudioDeviceTreeParser::createInputsArrayWithMuxes(){
                         
 		// do the detect parsing if it is one
         stopwordidx = 0;
-        NEXTENDOFWORD(thetempObject, stopwordidx);
+        NEXTENDOFWORD(thetempObject, stopwordidx, size);
                         
         if(!bcmp(thetempObject, kInputObjEntryName, stopwordidx-1)) {
             length = stopidx-startidx;
@@ -528,19 +677,19 @@ OSArray *AudioDeviceTreeParser::createInputsArrayWithMuxes(){
             
             while(stopwordidx < length) {
 
-                NEXTENDOFWORD(thetempObject, stopwordidx);
+                NEXTENDOFWORD(thetempObject, stopwordidx, size);
                 
                 if(!bcmp(thetempObject+startwordidx, kPortChannelsPropName, stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);
                     READWORDASNUMBER(myInfo.channels, thetempObject, startwordidx, stopwordidx);
                 } else if(!bcmp(thetempObject+startwordidx, kPortConnectionPropName, stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);
                     READWORDASNUMBER(myInfo.sndHWPort, thetempObject, startwordidx, stopwordidx);
                 } else if(!bcmp(thetempObject+startwordidx, kPortTypePropName, stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx); 
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size); 
                     READWORDASNUMBER(myInfo.inputPortType, thetempObject, startwordidx, stopwordidx);
                 }
                 ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);                                                 
@@ -567,11 +716,11 @@ OSArray *AudioDeviceTreeParser::createInputsArrayWithMuxes(){
 
              while(stopwordidx < length) {
 
-                NEXTENDOFWORD(thetempObject, stopwordidx);
+                NEXTENDOFWORD(thetempObject, stopwordidx, size);
                 
                 if(!bcmp(thetempObject+startwordidx, kModelPropName, stopwordidx-startwordidx)) { //model
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);
                     
                     if(!bcmp(thetempObject+startwordidx, kMuxProgOutName, 6)) 
                         myMuxInfo.MuxPortType = kAudioHardwareMuxPO;
@@ -584,7 +733,7 @@ OSArray *AudioDeviceTreeParser::createInputsArrayWithMuxes(){
                         
                 } else if(!bcmp(thetempObject+startwordidx, kSourceMapPropName, stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);
                   //  READWORDASNUMBER(myInfo.sndHWPort, thetempObject, startwordidx, stopwordidx);
                     tlength = sizeof(MuxSourceMap) * 2;
                     thetempObject[stopwordidx] = '\0';
@@ -592,7 +741,7 @@ OSArray *AudioDeviceTreeParser::createInputsArrayWithMuxes(){
                     thetempObject[stopwordidx] = ' ';
                 } else if(!bcmp(thetempObject+startwordidx, kSourceMapCountPropName, stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx); 
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size); 
                     READWORDASNUMBER(myMuxInfo.MuxPOnumSources, thetempObject, startwordidx, stopwordidx);
                 }                 
                 ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);                  
@@ -631,29 +780,28 @@ OSArray *AudioDeviceTreeParser::createDetectsArray(){
     myInfo.detectKind =0; myInfo.bitMask = 0;
     myInfo.bitMatch = 0; myInfo.device = 0;
 
-    
-            //get the detect numbers and create the detects array
+	// get the detect numbers and create the detects array
     AudioDetects = OSArray::withCapacity(getNumberOfDetects());
     
-            //get the complete Sound objects to parse them
+	// get the complete Sound objects to parse them
     tempData = OSDynamicCast(OSData, soundEntry->getProperty(kSoundObjectsPropName));
     FAIL_IF(!tempData, BAIL);
 
     theSoundObjects = (char *) tempData->getBytesNoCopy();
-    size = (int) tempData->getCapacity();
+    size = (int) tempData->getLength();
     parser = -1; detectidx =0;
     
-    while(parser < size) {
+    while(size && parser < size) {
 
-            //find the end of string
+		// find the end of string
         ASSIGNSTARTSTRING(startidx, parser);
         NEXTENDOFSTRING(theSoundObjects, parser);
         ASSIGNSTOPSTRING(stopidx, parser);
         thetempObject = theSoundObjects+startidx;
                         
-            //do the detect parsing if it is one
+		// do the detect parsing if it is one
         stopwordidx = 0;
-        NEXTENDOFWORD(thetempObject, stopwordidx);
+        NEXTENDOFWORD(thetempObject, stopwordidx, size);
                         
         if(!bcmp(thetempObject, kDetectObjEntryName, stopwordidx-1)) {
         
@@ -661,23 +809,23 @@ OSArray *AudioDeviceTreeParser::createDetectsArray(){
             startwordidx = ++stopwordidx;
             
             while(stopwordidx < length) {
-                NEXTENDOFWORD(thetempObject, stopwordidx);
+                NEXTENDOFWORD(thetempObject, stopwordidx, size);
     
                 if(!bcmp(thetempObject+startwordidx, kBitMaskPropName, stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);
                     READWORDASNUMBER(myInfo.bitMask, thetempObject, startwordidx, stopwordidx);
                 } else if(!bcmp(thetempObject+startwordidx, kBitMatchPropName, stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);
                     READWORDASNUMBER(myInfo.bitMatch, thetempObject, startwordidx, stopwordidx);
                 } else if(!bcmp(thetempObject+startwordidx, kDevicePropName, stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);
                     READWORDASNUMBER(myInfo.device, thetempObject, startwordidx, stopwordidx);
                 } else if(!bcmp(thetempObject+startwordidx, kModelPropName, stopwordidx-startwordidx)) {
                     ASSIGNNEXTWORD(thetempObject,startwordidx, stopwordidx);
-                    NEXTENDOFWORD(thetempObject, stopwordidx);
+                    NEXTENDOFWORD(thetempObject, stopwordidx, size);
                     if(!bcmp(thetempObject+startwordidx, kAnyInDetectObjName, 6))
                         myInfo.detectKind = kAudioHardwareDetectAnyInSense;
                     else if(!bcmp(thetempObject+startwordidx, kInSenseBitsDetectObjName, 6)) 
@@ -723,14 +871,13 @@ UInt8 AudioDeviceTreeParser::convertAsciiToHexData(char ascii) {
     return ascii-subvalue;
 }
 
-
 UInt32 AudioDeviceTreeParser::getStringAsNumber(char *string){
     SInt32 number, multiplier;
     char *valueStr;
     
     valueStr = string;
     number = 0;
-    if (('0' == *valueStr) && ('x' == *(valueStr+1))) { // this is an hexa
+    if (('0' == *valueStr) && ('x' == *(valueStr+1))) { // this is an hex number
         valueStr+=2;
         while(*valueStr)
             number = number*16+convertAsciiToHexData(*valueStr++);
