@@ -95,6 +95,10 @@ void AudioProj10PowerObject::setFullPowerState (void) {
 	}
 }
 
+void AudioProj10PowerObject::setHardwarePowerIdleOn ( void ) {
+    setHardwarePowerOn();
+}
+
 IOReturn AudioProj10PowerObject::setHardwarePowerOn(){
     DEBUG_IOLOG("+ AudioProj10PowerObject::setHardwarePowerOn\n");
     IOReturn result = kIOReturnSuccess;
@@ -154,7 +158,7 @@ IOReturn AudioProj10PowerObject::setHardwarePowerOff(){
         keyLargo->callPlatformFunction("keyLargo_writeRegUInt8", false, (void *)&gpioOffset, (void *)(UInt32)value, 0, 0);
     }
 
-    DEBUG_IOLOG("- AudioProj10PowerObject::setHardwarePowerOff");
+    DEBUG_IOLOG("- AudioProj10PowerObject::setHardwarePowerOff\n");
     return result;
 }
 
@@ -199,6 +203,10 @@ void AudioProj6PowerObject::setFullPowerState (void) {
 	}
 }
 
+void AudioProj6PowerObject::setHardwarePowerIdleOn ( void ) {
+    setHardwarePowerOn();
+}
+
 IOReturn AudioProj6PowerObject::setHardwarePowerOff(){
     IOReturn result = kIOReturnSuccess;
     IOService *HeathRow = 0;
@@ -324,6 +332,10 @@ void AudioProj8PowerObject::setFullPowerState (void) {
 	}
 }
 
+void AudioProj8PowerObject::setHardwarePowerIdleOn ( void ) {
+    setHardwarePowerOn();
+}
+
 IOReturn AudioProj8PowerObject::setHardwarePowerOff()
 {
     IOReturn result = kIOReturnSuccess;
@@ -424,10 +436,16 @@ IOReturn AudioProj7PowerObject::setHardwarePowerOff(){
         keyLargo->callPlatformFunction("keyLargo_writeRegUInt8", false, (void *)&gpioOffset, (void *)(UInt32)value, 0, 0);
     }
     
-    DEBUG_IOLOG("- AudioProj7PowerObject::setHardwarePowerOff");
+    DEBUG_IOLOG("- AudioProj7PowerObject::setHardwarePowerOff\n");
     return result;
 }
-	
+
+void AudioProj7PowerObject::setHardwarePowerIdleOn ( void ) {
+    DEBUG_IOLOG("+ AudioProj7PowerObject::setHardwarePowerIdleOn\n");
+    setHardwarePowerOn();
+    DEBUG_IOLOG("- AudioProj7PowerObject::setHardwarePowerIdleOn\n");
+}
+
 IOReturn AudioProj7PowerObject::setHardwarePowerOn(){
     DEBUG_IOLOG("+ AudioProj7PowerObject::setHardwarePowerOn\n");
     IOReturn result = kIOReturnSuccess;
@@ -498,6 +516,10 @@ void AudioProj14PowerObject::setFullPowerState (void) {
 	}
 }
 
+void AudioProj14PowerObject::setHardwarePowerIdleOn ( void ) {
+    setHardwarePowerOn();
+}
+
 IOReturn AudioProj14PowerObject::setHardwarePowerOff(){
     IOReturn result = kIOReturnSuccess;
 
@@ -574,6 +596,10 @@ void AudioProj16PowerObject::setFullPowerState (void) {
 	}
 }
 
+void AudioProj16PowerObject::setHardwarePowerIdleOn ( void ) {
+    setHardwarePowerOn();
+}
+
 IOReturn AudioProj16PowerObject::setHardwarePowerOff(){
     IOReturn result = kIOReturnSuccess;
 
